/*****************************************************************************
 * DMC_SCB_PRIO_Test_Core0.c
 *****************************************************************************/

/*This code runs two MDMA channels is parallel to read from the DMC. The DMC_PRIOx and DMC_PRIOMSKx registers
 * are programmed to elevate the priority of a particular MDMA channel to increase the DMC throughput for that channel

 */
#include <ccblkfn.h>
#include "adi_initialize.h"
#include "DMC_SCB_PRIO_Test.h"
#include <cdefBF707.h>
#include <defBF707.h>
#include "time.h"
#include "stdio.h"

#pragma align 32
section("L1_data_b") unsigned char ucIntDestBuff[MAX_SIZE];

#pragma align 32
section("l2_sram") unsigned char ucIntDestBuff1[MAX_SIZE];


volatile clock_t clock_start;
volatile clock_t clock_stop_mdma1;
volatile clock_t cycles_mdma1;
float throughput_mdma1;

volatile clock_t clock_stop_mdma2;
volatile clock_t cycles_mdma2;
float throughput_mdma2;

int DMA_Done_MDMA1=0;
int DMA_Done_MDMA2=0;

void MDMA1_Interrupt_Handler(uint32_t iid, void* handlerArg);
void MDMA2_Interrupt_Handler(uint32_t iid, void* handlerArg);


//Macro to configure MDMA
#define MDMA_CONFIG(S_MDMA_CHANNEL,S_CFG, S_ADDRSTART, S_MOD, S_COUNT,D_MDMA_CHANNEL,D_CFG, D_ADDRSTART, D_MOD, D_COUNT)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = S_CFG;\
		*pREG_DMA##S_MDMA_CHANNEL##_ADDRSTART = S_ADDRSTART;\
		*pREG_DMA##S_MDMA_CHANNEL##_XCNT = S_COUNT;\
		*pREG_DMA##S_MDMA_CHANNEL##_XMOD = S_MOD;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = D_CFG;\
		*pREG_DMA##D_MDMA_CHANNEL##_ADDRSTART = D_ADDRSTART;\
		*pREG_DMA##D_MDMA_CHANNEL##_XCNT = D_COUNT;\
		*pREG_DMA##D_MDMA_CHANNEL##_XMOD = D_MOD;


//Macro to wait for MDMA done
#define WAIT_FOR_DMADONE(DMA_CHANNEL)\
		while(!(*pREG_DMA##DMA_CHANNEL##_STAT & ENUM_DMA_STAT_IRQDONE));

//Macro to disable the MDMA
#define DISABLE_DMAs(S_MDMA_CHANNEL, D_MDMA_CHANNEL)\


int main(void)
{
	//DMA work unit size = 2048 bytes
	unsigned int uiBytes=2048;
	unsigned int uiBytes1=2048;


	*pREG_SPU0_SECUREP53=0x3;
	*pREG_SPU0_SECUREP54=0x3;
	*pREG_SPU0_SECUREP55=0x3;
	*pREG_SPU0_SECUREP56=0x3;

	*pREG_SCB0_MST30_SYNC=0;
	*pREG_SCB0_MST31_SYNC=0;
	*pREG_SCB0_IB7_SYNC=0;

	//*pREG_DMC0_PRIO=0x00C0;//DMA 18 high priority
	*pREG_DMC0_PRIO=0x100;//DMA 20 high priority
	*pREG_DMC0_PRIOMSK=0xFFFF;

	//Initialize and map the interrupts here
	*pREG_SEC0_GCTL=ENUM_SEC_GCTL_EN;
	*pREG_SEC0_CCTL0=ENUM_SEC_CCTL_EN;
	adi_int_InstallHandler(INTR_MDMA1_DST, MDMA1_Interrupt_Handler, NULL, true);
	adi_int_InstallHandler(INTR_MDMA2_DST, MDMA2_Interrupt_Handler, NULL, true);

	*pREG_DMA18_BWMCNT=0x7FFFFFFF;
	*pREG_DMA20_BWMCNT=0x7FFFFFFF;

	//Configure MDMA0 (18-19) to read from DMC
	MDMA_CONFIG(18, SOURCE_DMA_CFG, 0x80000000, SOURCE_MSIZE, uiBytes/SOURCE_MSIZE,19,DEST_DMA_CFG, ucIntDestBuff, DEST_MSIZE, uiBytes/DEST_MSIZE);

	//Configure MDMA2 (20-21) to read from DMC
	MDMA_CONFIG(20, SOURCE_DMA_CFG, 0x81000000, SOURCE_MSIZE, uiBytes1/SOURCE_MSIZE,21,DEST_DMA_CFG, ucIntDestBuff1, DEST_MSIZE, uiBytes1/DEST_MSIZE);


	//Enable the DMAs here
	*pREG_DMA19_CFG |= 0x1;
	*pREG_DMA21_CFG |= 0x1;
	*pREG_DMA18_CFG |= 0x1;
	*pREG_DMA20_CFG |= 0x1;

	//Start the cycle count
	clock_start = 0x7FFFFFFF;

	//Wait for the DMA done
	while(DMA_Done_MDMA1==0||DMA_Done_MDMA2==0);

	//Stop the cycle counts
	cycles_mdma1=clock_start-clock_stop_mdma1;
	cycles_mdma2=clock_start-clock_stop_mdma2;

	//Disable the DMAs
	DISABLE_DMAs(18,19);
	DISABLE_DMAs(20,21);

	//Calculate the throughput values
	throughput_mdma1=(float)(uiBytes*SCLK_SPEED/cycles_mdma1);
	throughput_mdma2=(float)(uiBytes*SCLK_SPEED/cycles_mdma2);
	printf("\n\n No of bytes=%d, No of cycles for MDMA1 =%d\n\n ",uiBytes,cycles_mdma1);
	printf("\n\n No of bytes=%d, No of cycles for MDMA2 =%d\n\n",uiBytes,cycles_mdma2);
	printf("Throughput MDMA1=%f MB/s, Throughput MDMA2=%f MB/s\n\n",throughput_mdma1, throughput_mdma2);

	return 0;
}


void MDMA1_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	//Stop the cycle count for MDMA0
	clock_stop_mdma1 =  *pREG_DMA18_BWMCNT_CUR;

	*pREG_DMA19_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Done_MDMA1 = 1;
}

void MDMA2_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	//Stop the cycle count for MDMA2
	clock_stop_mdma2 = *pREG_DMA20_BWMCNT_CUR;

	*pREG_DMA21_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Done_MDMA2 = 1;
}
